<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Login - doti18n studio</title>
    <link rel="stylesheet" href="/static/style.css">
</head>
<body class="login-page">
    <div class="login-box">
        <h1>doti18n studio</h1>
        <?py if error: ?>
            <p style="color: var(--danger-color);">Invalid username or password</p>
        <?py #endif ?>
        <form action="/login" method="POST">
            <input type="hidden" name="csrf_token" value="${csrf_token}">
            <label>
                <input type="text" name="username" placeholder="Username" required autofocus>
            </label>
            <label>
                <input type="password" name="password" placeholder="Password" required>
            </label>
            <button type="submit">Login</button>
        </form>
    </div>
</body>
</html>